
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/un.h>
#include <netdb.h>
#include <strings.h>
#include <netinet/in.h>
#include <errno.h>
#include <fcntl.h>
#include <stdio.h>
#ifndef MSG_CONFIRM
#define MSG_CONFIRM 0
#endif
#ifndef MSG_NOSIGNAL
#define MSG_NOSIGNAL 0
#endif
#ifndef MSG_DONTWAIT
#define MSG_DONTWAIT 0
#endif
#ifndef MSG_EOR
#define MSG_EOR 0
#endif

static void *
safe_buffer_pointer(cl_object x, cl_index size)
{
	cl_type t = type_of(x);
	int ok = 0;
	if (t == t_base_string) {
		ok = (size < x->base_string.dim);
	} else if (t == t_vector) {
		cl_elttype aet = (cl_elttype)x->vector.elttype;
		if (aet == aet_b8 || aet == aet_i8 || aet == aet_bc) {
			ok = (size < x->vector.dim);
		} else if (aet == aet_fix || aet == aet_index) {
			size /= sizeof(cl_index);
			ok = (size < x->vector.dim);
		}
	}
	if (!ok) {
		FEerror("Lisp object is not a valid socket buffer: ~A", 1, x);
	}
	return (void *)x->vector.self.ch;
}


static void fill_inet_sockaddr(struct sockaddr_in *sockaddr, int port,
			       int a1, int a2, int a3, int a4)
{
#if defined(_MSC_VER) || defined(mingw32)
	memset(sockaddr,0,sizeof(struct sockaddr_in));
#else
	bzero(sockaddr,sizeof(struct sockaddr_in));
#endif
	sockaddr->sin_family = AF_INET;
	sockaddr->sin_port = htons(port);
	sockaddr->sin_addr.s_addr= htonl((uint32_t)a1<<24 | (uint32_t)a2<<16 | (uint32_t)a3<<8 | (uint32_t)a4) ;

}

#ifdef __cplusplus
extern "C" {
#endif
static cl_object LC1define_c_constants(cl_object , cl_object );
#define VT2 cl_object T0,T1,T2;
#define VLEX2
#define CLSR2
static cl_object LC2c_constant(cl_object , cl_object );
#ifdef __cplusplus
extern cl_object si_dm_too_few_arguments(...);
#else
extern cl_object si_dm_too_few_arguments();
#endif
#ifdef __cplusplus
extern cl_object si_check_arg_length(...);
#else
extern cl_object si_check_arg_length();
#endif
#define VT3
#define VLEX3
#define CLSR3
static cl_object L3ff_socket(cl_object , cl_object , cl_object );
#define VT4
#define VLEX4
#define CLSR4
static cl_object L4ff_listen(cl_object , cl_object );
#define VT5
#define VLEX5
#define CLSR5
static cl_object L5ff_close(cl_object );
#define VT6
#define VLEX6
#define CLSR6
static cl_object L7split(cl_narg, cl_object , ...);
#define VT7 cl_object T0;
#define VLEX7
#define CLSR7 volatile cl_object env0; cl_object CLV0;
static cl_object LC6is_ws(cl_narg, cl_object , cl_object , ...);
#define VT8
#define VLEX8
#define CLSR8 cl_object CLV0;
#ifdef __cplusplus
extern cl_object clos_ensure_class(...);
#else
extern cl_object clos_ensure_class();
#endif
#ifdef __cplusplus
extern cl_object clos_install_method(...);
#else
extern cl_object clos_install_method();
#endif
static cl_object LC8host_ent_address(cl_object );
#define VT9 cl_object T0;
#define VLEX9
#define CLSR9
static cl_object L9get_host_by_name(cl_object );
#define VT10 cl_object T0,T1,T2,T3;
#define VLEX10
#define CLSR10
static cl_object L12get_host_by_address(cl_object );
#define VT11 cl_object T0,T1,T2,T3;
#define VLEX11
#define CLSR11 volatile cl_object env0; cl_object CLV0,CLV1;
static cl_object LC10NIL(cl_narg, cl_object , ...);
#define VT12
#define VLEX12
#define CLSR12 cl_object CLV0,CLV1;
static cl_object LC11NIL(cl_object );
#define VT13
#define VLEX13
#define CLSR13
static cl_object LC13NIL();
#define VT14
#define VLEX14
#define CLSR14
static cl_object LC15print_object(cl_object , cl_object );
#define VT15
#define VLEX15
#define CLSR15 volatile cl_object env0; cl_object CLV0,CLV1;
static cl_object LC14si___print_unreadable_object_body_(cl_narg, cl_object , ...);
#ifdef __cplusplus
extern cl_object cl_slot_value(...);
#else
extern cl_object cl_slot_value();
#endif
#define VT16 cl_object T0;
#define VLEX16
#define CLSR16 cl_object CLV0,CLV1;
static cl_object LC16shared_initialize(cl_narg, cl_object , cl_object , ...);
#define LC16shared_initializekeys (&VV[179])
#ifdef __cplusplus
extern cl_object cl_slot_boundp(...);
#else
extern cl_object cl_slot_boundp();
#endif
#ifdef __cplusplus
extern cl_object si_ecase_error(...);
#else
extern cl_object si_ecase_error();
#endif
#define VT17 cl_object T0,T1;
#define VLEX17
#define CLSR17
static cl_object LC17socket_listen(cl_object , cl_object );
#define VT18 cl_object T0;
#define VLEX18
#define CLSR18
static cl_object LC18socket_close_low_level(cl_object );
#define VT19 cl_object T0;
#define VLEX19
#define CLSR19
static cl_object LC19socket_close(cl_object );
#ifdef __cplusplus
extern cl_object cl_slot_makunbound(...);
#else
extern cl_object cl_slot_makunbound();
#endif
#define VT20 cl_object T0;
#define VLEX20
#define CLSR20
static cl_object LC20socket_receive(cl_narg, cl_object , cl_object , cl_object , ...);
#define LC20socket_receivekeys (&VV[185])
#define VT21
#define VLEX21
#define CLSR21
static cl_object L21get_protocol_by_name(cl_object );
#define VT22
#define VLEX22
#define CLSR22
static cl_object L22make_inet_address(cl_object );
#define VT23 cl_object T0,T1;
#define VLEX23
#define CLSR23
static cl_object LC23NIL();
#define VT24
#define VLEX24
#define CLSR24
static cl_object L24make_inet_socket(cl_object , cl_object );
#define VT25
#define VLEX25
#define CLSR25
static cl_object LC27socket_bind(cl_narg, cl_object , ...);
#define VT26 cl_object T0,T1,T2;
#define VLEX26
#define CLSR26 volatile cl_object env0; cl_object CLV0,CLV1;
static cl_object LC25NIL(cl_narg, cl_object , ...);
#define VT27
#define VLEX27
#define CLSR27 cl_object CLV0,CLV1;
static cl_object LC26NIL(cl_object );
#define VT28
#define VLEX28
#define CLSR28
static cl_object LC28socket_accept(cl_object );
#define VT29 cl_object T0,T1,T2,T3;
#define VLEX29
#define CLSR29
static cl_object LC29socket_connect(cl_narg, cl_object , ...);
#define VT30 cl_object T0,T1;
#define VLEX30
#define CLSR30
static cl_object LC30socket_peername(cl_object );
#define VT31
#define VLEX31
#define CLSR31
static cl_object LC31socket_name(cl_object );
#define VT32
#define VLEX32
#define CLSR32
static cl_object LC36socket_send(cl_narg, cl_object , cl_object , cl_object , ...);
#define LC36socket_sendkeys (&VV[191])
#define VT33 cl_object T0,T1,T2,T3,T4;
#define VLEX33
#define CLSR33 volatile cl_object env0; cl_object CLV0,CLV1;
static cl_object LC32NIL(cl_narg, cl_object , ...);
#define VT34
#define VLEX34
#define CLSR34 cl_object CLV0,CLV1;
static cl_object LC33NIL(cl_object );
#define VT35
#define VLEX35
#define CLSR35
static cl_object LC34NIL(cl_narg, cl_object , ...);
#define VT36
#define VLEX36
#define CLSR36 cl_object CLV0,CLV1;
static cl_object LC35NIL(cl_object );
#define VT37
#define VLEX37
#define CLSR37
static cl_object LC37NIL();
#define VT38
#define VLEX38
#define CLSR38
static cl_object LC40socket_bind(cl_narg, cl_object , ...);
#define VT39 cl_object T0,T1,T2;
#define VLEX39
#define CLSR39 volatile cl_object env0; cl_object CLV0,CLV1;
static cl_object LC41socket_accept(cl_object );
#define VT40 cl_object T0,T1,T2,T3;
#define VLEX40
#define CLSR40
static cl_object LC44socket_connect(cl_narg, cl_object , ...);
#define VT41 cl_object T0,T1,T2;
#define VLEX41
#define CLSR41 volatile cl_object env0; cl_object CLV0,CLV1;
static cl_object LC45socket_peername(cl_object );
#define VT42
#define VLEX42
#define CLSR42
static cl_object LC38NIL(cl_narg, cl_object , ...);
#define VT43
#define VLEX43
#define CLSR43 cl_object CLV0,CLV1;
static cl_object LC39NIL(cl_object );
#define VT44
#define VLEX44
#define CLSR44
static cl_object LC42NIL(cl_narg, cl_object , ...);
#define VT45
#define VLEX45
#define CLSR45 cl_object CLV0,CLV1;
static cl_object LC43NIL(cl_object );
#define VT46
#define VLEX46
#define CLSR46
static cl_object LC46non_blocking_mode(cl_object );
#define VT47 cl_object T0;
#define VLEX47
#define CLSR47
static cl_object LC47_setf_non_blocking_mode_(cl_object , cl_object );
#define VT48 cl_object T0;
#define VLEX48
#define CLSR48
static cl_object L50make_stream_from_fd(cl_narg, cl_object , cl_object , cl_object , ...);
#define VT49 cl_object T0,T1,T2;
#define VLEX49
#define CLSR49 volatile cl_object env0; cl_object CLV0,CLV1;
static cl_object LC48NIL(cl_narg, cl_object , ...);
#define VT50
#define VLEX50
#define CLSR50 cl_object CLV0,CLV1;
static cl_object LC49NIL(cl_object );
#define VT51
#define VLEX51
#define CLSR51
static cl_object LC51socket_make_stream(cl_narg, cl_object , ...);
#define LC51socket_make_streamkeys (&VV[200])
#define VT52
#define VLEX52
#define CLSR52
static cl_object LC53print_object(cl_object , cl_object );
#define VT53 cl_object T0,T1,T2;
#define VLEX53 cl_object lex0[3];
#define CLSR53
static cl_object LC52call_next_method(cl_narg, cl_object *, ...);
#define VT54
#define VLEX54
#define CLSR54
static cl_object LC54define_socket_condition(cl_object , cl_object );
#define VT55 cl_object T0,T1,T2,T3,T4,T5,T6,T7,T8,T9,T10;
#define VLEX55
#define CLSR55
static cl_object L55condition_for_errno(cl_object );
#define VT56 cl_object T0;
#define VLEX56
#define CLSR56
static cl_object L56socket_error(cl_object );
#define VT57
#define VLEX57
#define CLSR57
static cl_object L57name_service_error(cl_object );
#define VT58 cl_object T0;
#define VLEX58
#define CLSR58
static cl_object LC59print_object(cl_object , cl_object );
#define VT59 cl_object T0,T1,T2;
#define VLEX59 cl_object lex0[3];
#define CLSR59
static cl_object LC58call_next_method(cl_narg, cl_object *, ...);
#define VT60
#define VLEX60
#define CLSR60
static cl_object LC60define_name_service_condition(cl_object , cl_object );
#define VT61 cl_object T0,T1,T2,T3,T4,T5,T6,T7,T8,T9,T10;
#define VLEX61
#define CLSR61
static cl_object L61condition_for_name_service_errno(cl_object );
#define VT62 cl_object T0;
#define VLEX62
#define CLSR62
static cl_object L62get_name_service_errno();
#define VT63
#define VLEX63
#define CLSR63
static cl_object L63get_name_service_error_message(cl_object );
#define VT64
#define VLEX64
#define CLSR64
static cl_object L64get_sockopt_int(cl_object , cl_object );
#define VT65 cl_object T0;
#define VLEX65
#define CLSR65
static cl_object L65get_sockopt_bool(cl_object , cl_object );
#define VT66 cl_object T0;
#define VLEX66
#define CLSR66
static cl_object L66set_sockopt_int(cl_object , cl_object , cl_object );
#define VT67 cl_object T0;
#define VLEX67
#define CLSR67
static cl_object L67set_sockopt_bool(cl_object , cl_object , cl_object );
#define VT68 cl_object T0;
#define VLEX68
#define CLSR68
static cl_object LC68define_sockopt(cl_object , cl_object );
#define VT69 cl_object T0,T1,T2,T3,T4,T5,T6,T7,T8,T9,T10,T11,T12,T13;
#define VLEX69
#define CLSR69
static cl_object L69sockopt_type(cl_object );
#define VT70 cl_object T0,T1;
#define VLEX70
#define CLSR70
static cl_object L70sockopt_receive_buffer(cl_object );
#define VT71 cl_object T0,T1;
#define VLEX71
#define CLSR71
static cl_object L71_setf_sockopt_receive_buffer_(cl_object , cl_object );
#define VT72 cl_object T0,T1;
#define VLEX72
#define CLSR72
static cl_object L72sockopt_reuse_address(cl_object );
#define VT73 cl_object T0,T1;
#define VLEX73
#define CLSR73
static cl_object L73_setf_sockopt_reuse_address_(cl_object , cl_object );
#define VT74 cl_object T0,T1;
#define VLEX74
#define CLSR74
static cl_object L74sockopt_keep_alive(cl_object );
#define VT75 cl_object T0,T1;
#define VLEX75
#define CLSR75
static cl_object L75_setf_sockopt_keep_alive_(cl_object , cl_object );
#define VT76 cl_object T0,T1;
#define VLEX76
#define CLSR76
static cl_object L76socket_dont_route(cl_object );
#define VT77 cl_object T0,T1;
#define VLEX77
#define CLSR77
static cl_object L77_setf_socket_dont_route_(cl_object , cl_object );
#define VT78 cl_object T0,T1;
#define VLEX78
#define CLSR78
static cl_object L78socket_linger(cl_object );
#define VT79 cl_object T0,T1;
#define VLEX79
#define CLSR79
static cl_object L79_setf_socket_linger_(cl_object , cl_object );
#define VT80 cl_object T0,T1;
#define VLEX80
#define CLSR80
#define VT1 cl_object T0,T1,T2,T3,T4,T5;
#define VLEX1
#define CLSR1
static cl_object Cblock;
#define VM1 6
#define VM80 2
#define VM79 2
#define VM78 2
#define VM77 2
#define VM76 2
#define VM75 2
#define VM74 2
#define VM73 2
#define VM72 2
#define VM71 2
#define VM70 2
#define VM69 14
#define VM68 1
#define VM67 1
#define VM66 1
#define VM65 1
#define VM64 0
#define VM63 0
#define VM62 1
#define VM61 11
#define VM60 0
#define VM59 3
#define VM58 1
#define VM57 0
#define VM56 1
#define VM55 11
#define VM54 0
#define VM53 3
#define VM52 0
#define VM51 0
#define VM50 0
#define VM49 3
#define VM48 1
#define VM47 1
#define VM46 0
#define VM45 0
#define VM44 0
#define VM43 0
#define VM42 0
#define VM41 3
#define VM40 4
#define VM39 3
#define VM38 0
#define VM37 0
#define VM36 0
#define VM35 0
#define VM34 0
#define VM33 5
#define VM32 0
#define VM31 0
#define VM30 2
#define VM29 4
#define VM28 0
#define VM27 0
#define VM26 3
#define VM25 0
#define VM24 0
#define VM23 2
#define VM22 0
#define VM21 0
#define VM20 1
#define VM19 1
#define VM18 1
#define VM17 2
#define VM16 1
#define VM15 0
#define VM14 0
#define VM13 0
#define VM12 0
#define VM11 4
#define VM10 4
#define VM9 1
#define VM8 0
#define VM7 1
#define VM6 0
#define VM5 0
#define VM4 0
#define VM3 0
#define VM2 3
#define VM 205
#define VMtemp 80
#ifdef ECL_DYNAMIC_VV
static cl_object *VV;
#else
static cl_object VV[VM];
#endif
static cl_object LKF18name_service_error_symbol(cl_narg, ...);
static cl_object (*LK18name_service_error_symbol)(cl_narg, ...)=LKF18name_service_error_symbol;
static cl_object LKF17name_service_error_syscall(cl_narg, ...);
static cl_object (*LK17name_service_error_syscall)(cl_narg, ...)=LKF17name_service_error_syscall;
static cl_object LKF16name_service_error_errno(cl_narg, ...);
static cl_object (*LK16name_service_error_errno)(cl_narg, ...)=LKF16name_service_error_errno;
static cl_object LKF15socket_error_symbol(cl_narg, ...);
static cl_object (*LK15socket_error_symbol)(cl_narg, ...)=LKF15socket_error_symbol;
static cl_object LKF14socket_error_syscall(cl_narg, ...);
static cl_object (*LK14socket_error_syscall)(cl_narg, ...)=LKF14socket_error_syscall;
static cl_object LKF13socket_error_errno(cl_narg, ...);
static cl_object (*LK13socket_error_errno)(cl_narg, ...)=LKF13socket_error_errno;
static cl_object LKF12socket_protocol(cl_narg, ...);
static cl_object (*LK12socket_protocol)(cl_narg, ...)=LKF12socket_protocol;
static cl_object LKF11si__coerce_to_condition(cl_narg, ...);
static cl_object (*LK11si__coerce_to_condition)(cl_narg, ...)=LKF11si__coerce_to_condition;
static cl_object LKF10socket_close_low_level(cl_narg, ...);
static cl_object (*LK10socket_close_low_level)(cl_narg, ...)=LKF10socket_close_low_level;
static cl_object LKF9socket_type(cl_narg, ...);
static cl_object (*LK9socket_type)(cl_narg, ...)=LKF9socket_type;
static cl_object LKF8socket_family(cl_narg, ...);
static cl_object (*LK8socket_family)(cl_narg, ...)=LKF8socket_family;
static cl_object LKF7socket_file_descriptor(cl_narg, ...);
static cl_object (*LK7socket_file_descriptor)(cl_narg, ...)=LKF7socket_file_descriptor;
static cl_object LKF6si__print_unreadable_object_function(cl_narg, ...);
static cl_object (*LK6si__print_unreadable_object_function)(cl_narg, ...)=LKF6si__print_unreadable_object_function;
static cl_object LKF5si__assert_report(cl_narg, ...);
static cl_object (*LK5si__assert_report)(cl_narg, ...)=LKF5si__assert_report;
static cl_object LKF4si__assert_prompt(cl_narg, ...);
static cl_object (*LK4si__assert_prompt)(cl_narg, ...)=LKF4si__assert_prompt;
static cl_object LKF3si__simple_assertion_failure(cl_narg, ...);
static cl_object (*LK3si__simple_assertion_failure)(cl_narg, ...)=LKF3si__simple_assertion_failure;
static cl_object LKF2si__make_restart(cl_narg, ...);
static cl_object (*LK2si__make_restart)(cl_narg, ...)=LKF2si__make_restart;
static cl_object LKF1make_instance(cl_narg, ...);
static cl_object (*LK1make_instance)(cl_narg, ...)=LKF1make_instance;
static cl_object LKF0host_ent_addresses(cl_narg, ...);
static cl_object (*LK0host_ent_addresses)(cl_narg, ...)=LKF0host_ent_addresses;
#ifdef __cplusplus
}
#endif
